/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

// @(#) $Id: IListDomWM.java 10268 2013-01-11 10:34:57Z dschwarz $
package at.spardat.xma.mdl.list;

import at.spardat.xma.mdl.IFormattable;
import at.spardat.xma.mdl.ISelectable;

/**
 * The widget model that encapsulates domain values which are usually displayed in
 * a combo box. <p>
 * 
 * The displayed entries in the combobox are drawn from a tabular data source, 
 * see {@link at.spardat.xma.datasource.ITabularDataSource ITabularDataSource}.
 *  
 * @author YSD, 23.06.2003 12:57:05
 */
public interface IListDomWM extends ISelectable, IFormattable {
    
    /**
     * Sets a new data source. If <tt>null</tt> is provided, 
     * an empty table is considered to be the range of values.
     * 
     * @param dataSource specification of a tabular data source.
     * @see at.spardat.xma.datasource.ITabularDataSource
     */
    public abstract void setDataSource (String dataSource);
    
    /**
     * Returns the set data source or null, if non is set.
     * 
     * @return data source specification
     * @see at.spardat.xma.datasource.ITabularDataSource
     */
    public abstract String getDataSource();
    
    /**
     * Defines whether the data-source is defined at compile-time in the UI-designer
     * or is always set dynamically (via <tt>setDataSource()</tt>).
     */
    public boolean isDynamicDataSource ();
    
    /**
     * Sets the selection of this ListDomWM to the given value.
     * If value is empty (null or ""), the selection is cleared.
     * @param value to select
     */
    public void set(String value);

}
